# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from .queues import (
    QueuesResource,
    AsyncQueuesResource,
    QueuesResourceWithRawResponse,
    AsyncQueuesResourceWithRawResponse,
    QueuesResourceWithStreamingResponse,
    AsyncQueuesResourceWithStreamingResponse,
)

__all__ = [
    "CallsResource",
    "AsyncCallsResource",
    "CallsResourceWithRawResponse",
    "AsyncCallsResourceWithRawResponse",
    "CallsResourceWithStreamingResponse",
    "AsyncCallsResourceWithStreamingResponse",
    "QueuesResource",
    "AsyncQueuesResource",
    "QueuesResourceWithRawResponse",
    "AsyncQueuesResourceWithRawResponse",
    "QueuesResourceWithStreamingResponse",
    "AsyncQueuesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ...types import (
    external_connection_list_params,
    external_connection_create_params,
    external_connection_update_params,
    external_connection_update_location_params,
)
from .uploads import (
    UploadsResource,
    AsyncUploadsResource,
    UploadsResourceWithRawResponse,
    AsyncUploadsResourceWithRawResponse,
    UploadsResourceWithStreamingResponse,
    AsyncUploadsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .releases import (
    ReleasesResource,
    AsyncReleasesResource,
    ReleasesResourceWithRawResponse,
    AsyncReleasesResourceWithRawResponse,
    ReleasesResourceWithStreamingResponse,
    AsyncReleasesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .log_messages import (
    LogMessagesResource,
    AsyncLogMessagesResource,
    LogMessagesResourceWithRawResponse,
    AsyncLogMessagesResourceWithRawResponse,
    LogMessagesResourceWithStreamingResponse,
    AsyncLogMessagesResourceWithStreamingResponse,
)
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from .civic_addresses import (
    CivicAddressesResource,
    AsyncCivicAddressesResource,
    CivicAddressesResourceWithRawResponse,
    AsyncCivicAddressesResourceWithRawResponse,
    CivicAddressesResourceWithStreamingResponse,
    AsyncCivicAddressesResourceWithStreamingResponse,
)
from ...types.external_connection import ExternalConnection
from ...types.external_connection_create_response import ExternalConnectionCreateResponse
from ...types.external_connection_delete_response import ExternalConnectionDeleteResponse
from ...types.external_connection_update_response import ExternalConnectionUpdateResponse
from ...types.external_connection_retrieve_response import ExternalConnectionRetrieveResponse
from ...types.external_connection_update_location_response import ExternalConnectionUpdateLocationResponse

__all__ = ["ExternalConnectionsResource", "AsyncExternalConnectionsResource"]


class ExternalConnectionsResource(SyncAPIResource):
    @cached_property
    def log_messages(self) -> LogMessagesResource:
        return LogMessagesResource(self._client)

    @cached_property
    def civic_addresses(self) -> CivicAddressesResource:
        return CivicAddressesResource(self._client)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResource:
        return PhoneNumbersResource(self._client)

    @cached_property
    def releases(self) -> ReleasesResource:
        return ReleasesResource(self._client)

    @cached_property
    def uploads(self) -> UploadsResource:
        return UploadsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ExternalConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ExternalConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExternalConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ExternalConnectionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        external_sip_connection: Literal["zoom"],
        outbound: external_connection_create_params.Outbound,
        active: bool | Omit = omit,
        inbound: external_connection_create_params.Inbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionCreateResponse:
        """
        Creates a new External Connection based on the parameters sent in the request.
        The external_sip_connection and outbound voice profile id are required. Once
        created, you can assign phone numbers to your application using the
        `/phone_numbers` endpoint.

        Args:
          external_sip_connection: The service that will be consuming this connection.

          active: Specifies whether the connection can be used.

          tags: Tags associated with the connection.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/external_connections",
            body=maybe_transform(
                {
                    "external_sip_connection": external_sip_connection,
                    "outbound": outbound,
                    "active": active,
                    "inbound": inbound,
                    "tags": tags,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                external_connection_create_params.ExternalConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionRetrieveResponse:
        """
        Return the details of an existing External Connection inside the 'data'
        attribute of the response.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/external_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        outbound: external_connection_update_params.Outbound,
        active: bool | Omit = omit,
        inbound: external_connection_update_params.Inbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionUpdateResponse:
        """
        Updates settings of an existing External Connection based on the parameters of
        the request.

        Args:
          active: Specifies whether the connection can be used.

          tags: Tags associated with the connection.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/external_connections/{id}",
            body=maybe_transform(
                {
                    "outbound": outbound,
                    "active": active,
                    "inbound": inbound,
                    "tags": tags,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                external_connection_update_params.ExternalConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: external_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ExternalConnection]:
        """
        This endpoint returns a list of your External Connections inside the 'data'
        attribute of the response. External Connections are used by Telnyx customers to
        seamless configure SIP trunking integrations with Telnyx Partners, through
        External Voice Integrations in Mission Control Portal.

        Args:
          filter: Filter parameter for external connections (deepObject style). Supports filtering
              by connection_name, external_sip_connection, id, created_at, and phone_number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/external_connections",
            page=SyncDefaultFlatPagination[ExternalConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    external_connection_list_params.ExternalConnectionListParams,
                ),
            ),
            model=ExternalConnection,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionDeleteResponse:
        """Permanently deletes an External Connection.

        Deletion may be prevented if the
        application is in use by phone numbers, is active, or if it is an Operator
        Connect connection. To remove an Operator Connect integration please contact
        Telnyx support.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/external_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionDeleteResponse,
        )

    def update_location(
        self,
        location_id: str,
        *,
        id: str,
        static_emergency_address_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionUpdateLocationResponse:
        """
        Update a location's static emergency address

        Args:
          static_emergency_address_id: A new static emergency address ID to update the location with

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not location_id:
            raise ValueError(f"Expected a non-empty value for `location_id` but received {location_id!r}")
        return self._patch(
            f"/external_connections/{id}/locations/{location_id}",
            body=maybe_transform(
                {"static_emergency_address_id": static_emergency_address_id},
                external_connection_update_location_params.ExternalConnectionUpdateLocationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionUpdateLocationResponse,
        )


class AsyncExternalConnectionsResource(AsyncAPIResource):
    @cached_property
    def log_messages(self) -> AsyncLogMessagesResource:
        return AsyncLogMessagesResource(self._client)

    @cached_property
    def civic_addresses(self) -> AsyncCivicAddressesResource:
        return AsyncCivicAddressesResource(self._client)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResource:
        return AsyncPhoneNumbersResource(self._client)

    @cached_property
    def releases(self) -> AsyncReleasesResource:
        return AsyncReleasesResource(self._client)

    @cached_property
    def uploads(self) -> AsyncUploadsResource:
        return AsyncUploadsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncExternalConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExternalConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExternalConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncExternalConnectionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        external_sip_connection: Literal["zoom"],
        outbound: external_connection_create_params.Outbound,
        active: bool | Omit = omit,
        inbound: external_connection_create_params.Inbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionCreateResponse:
        """
        Creates a new External Connection based on the parameters sent in the request.
        The external_sip_connection and outbound voice profile id are required. Once
        created, you can assign phone numbers to your application using the
        `/phone_numbers` endpoint.

        Args:
          external_sip_connection: The service that will be consuming this connection.

          active: Specifies whether the connection can be used.

          tags: Tags associated with the connection.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/external_connections",
            body=await async_maybe_transform(
                {
                    "external_sip_connection": external_sip_connection,
                    "outbound": outbound,
                    "active": active,
                    "inbound": inbound,
                    "tags": tags,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                external_connection_create_params.ExternalConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionRetrieveResponse:
        """
        Return the details of an existing External Connection inside the 'data'
        attribute of the response.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/external_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        outbound: external_connection_update_params.Outbound,
        active: bool | Omit = omit,
        inbound: external_connection_update_params.Inbound | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        webhook_event_failover_url: Optional[str] | Omit = omit,
        webhook_event_url: str | Omit = omit,
        webhook_timeout_secs: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionUpdateResponse:
        """
        Updates settings of an existing External Connection based on the parameters of
        the request.

        Args:
          active: Specifies whether the connection can be used.

          tags: Tags associated with the connection.

          webhook_event_failover_url: The failover URL where webhooks related to this connection will be sent if
              sending to the primary URL fails. Must include a scheme, such as 'https'.

          webhook_event_url: The URL where webhooks related to this connection will be sent. Must include a
              scheme, such as 'https'.

          webhook_timeout_secs: Specifies how many seconds to wait before timing out a webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/external_connections/{id}",
            body=await async_maybe_transform(
                {
                    "outbound": outbound,
                    "active": active,
                    "inbound": inbound,
                    "tags": tags,
                    "webhook_event_failover_url": webhook_event_failover_url,
                    "webhook_event_url": webhook_event_url,
                    "webhook_timeout_secs": webhook_timeout_secs,
                },
                external_connection_update_params.ExternalConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionUpdateResponse,
        )

    def list(
        self,
        *,
        filter: external_connection_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ExternalConnection, AsyncDefaultFlatPagination[ExternalConnection]]:
        """
        This endpoint returns a list of your External Connections inside the 'data'
        attribute of the response. External Connections are used by Telnyx customers to
        seamless configure SIP trunking integrations with Telnyx Partners, through
        External Voice Integrations in Mission Control Portal.

        Args:
          filter: Filter parameter for external connections (deepObject style). Supports filtering
              by connection_name, external_sip_connection, id, created_at, and phone_number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/external_connections",
            page=AsyncDefaultFlatPagination[ExternalConnection],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    external_connection_list_params.ExternalConnectionListParams,
                ),
            ),
            model=ExternalConnection,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionDeleteResponse:
        """Permanently deletes an External Connection.

        Deletion may be prevented if the
        application is in use by phone numbers, is active, or if it is an Operator
        Connect connection. To remove an Operator Connect integration please contact
        Telnyx support.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/external_connections/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionDeleteResponse,
        )

    async def update_location(
        self,
        location_id: str,
        *,
        id: str,
        static_emergency_address_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExternalConnectionUpdateLocationResponse:
        """
        Update a location's static emergency address

        Args:
          static_emergency_address_id: A new static emergency address ID to update the location with

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not location_id:
            raise ValueError(f"Expected a non-empty value for `location_id` but received {location_id!r}")
        return await self._patch(
            f"/external_connections/{id}/locations/{location_id}",
            body=await async_maybe_transform(
                {"static_emergency_address_id": static_emergency_address_id},
                external_connection_update_location_params.ExternalConnectionUpdateLocationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExternalConnectionUpdateLocationResponse,
        )


class ExternalConnectionsResourceWithRawResponse:
    def __init__(self, external_connections: ExternalConnectionsResource) -> None:
        self._external_connections = external_connections

        self.create = to_raw_response_wrapper(
            external_connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            external_connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            external_connections.update,
        )
        self.list = to_raw_response_wrapper(
            external_connections.list,
        )
        self.delete = to_raw_response_wrapper(
            external_connections.delete,
        )
        self.update_location = to_raw_response_wrapper(
            external_connections.update_location,
        )

    @cached_property
    def log_messages(self) -> LogMessagesResourceWithRawResponse:
        return LogMessagesResourceWithRawResponse(self._external_connections.log_messages)

    @cached_property
    def civic_addresses(self) -> CivicAddressesResourceWithRawResponse:
        return CivicAddressesResourceWithRawResponse(self._external_connections.civic_addresses)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResourceWithRawResponse:
        return PhoneNumbersResourceWithRawResponse(self._external_connections.phone_numbers)

    @cached_property
    def releases(self) -> ReleasesResourceWithRawResponse:
        return ReleasesResourceWithRawResponse(self._external_connections.releases)

    @cached_property
    def uploads(self) -> UploadsResourceWithRawResponse:
        return UploadsResourceWithRawResponse(self._external_connections.uploads)


class AsyncExternalConnectionsResourceWithRawResponse:
    def __init__(self, external_connections: AsyncExternalConnectionsResource) -> None:
        self._external_connections = external_connections

        self.create = async_to_raw_response_wrapper(
            external_connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            external_connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            external_connections.update,
        )
        self.list = async_to_raw_response_wrapper(
            external_connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            external_connections.delete,
        )
        self.update_location = async_to_raw_response_wrapper(
            external_connections.update_location,
        )

    @cached_property
    def log_messages(self) -> AsyncLogMessagesResourceWithRawResponse:
        return AsyncLogMessagesResourceWithRawResponse(self._external_connections.log_messages)

    @cached_property
    def civic_addresses(self) -> AsyncCivicAddressesResourceWithRawResponse:
        return AsyncCivicAddressesResourceWithRawResponse(self._external_connections.civic_addresses)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        return AsyncPhoneNumbersResourceWithRawResponse(self._external_connections.phone_numbers)

    @cached_property
    def releases(self) -> AsyncReleasesResourceWithRawResponse:
        return AsyncReleasesResourceWithRawResponse(self._external_connections.releases)

    @cached_property
    def uploads(self) -> AsyncUploadsResourceWithRawResponse:
        return AsyncUploadsResourceWithRawResponse(self._external_connections.uploads)


class ExternalConnectionsResourceWithStreamingResponse:
    def __init__(self, external_connections: ExternalConnectionsResource) -> None:
        self._external_connections = external_connections

        self.create = to_streamed_response_wrapper(
            external_connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            external_connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            external_connections.update,
        )
        self.list = to_streamed_response_wrapper(
            external_connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            external_connections.delete,
        )
        self.update_location = to_streamed_response_wrapper(
            external_connections.update_location,
        )

    @cached_property
    def log_messages(self) -> LogMessagesResourceWithStreamingResponse:
        return LogMessagesResourceWithStreamingResponse(self._external_connections.log_messages)

    @cached_property
    def civic_addresses(self) -> CivicAddressesResourceWithStreamingResponse:
        return CivicAddressesResourceWithStreamingResponse(self._external_connections.civic_addresses)

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResourceWithStreamingResponse:
        return PhoneNumbersResourceWithStreamingResponse(self._external_connections.phone_numbers)

    @cached_property
    def releases(self) -> ReleasesResourceWithStreamingResponse:
        return ReleasesResourceWithStreamingResponse(self._external_connections.releases)

    @cached_property
    def uploads(self) -> UploadsResourceWithStreamingResponse:
        return UploadsResourceWithStreamingResponse(self._external_connections.uploads)


class AsyncExternalConnectionsResourceWithStreamingResponse:
    def __init__(self, external_connections: AsyncExternalConnectionsResource) -> None:
        self._external_connections = external_connections

        self.create = async_to_streamed_response_wrapper(
            external_connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            external_connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            external_connections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            external_connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            external_connections.delete,
        )
        self.update_location = async_to_streamed_response_wrapper(
            external_connections.update_location,
        )

    @cached_property
    def log_messages(self) -> AsyncLogMessagesResourceWithStreamingResponse:
        return AsyncLogMessagesResourceWithStreamingResponse(self._external_connections.log_messages)

    @cached_property
    def civic_addresses(self) -> AsyncCivicAddressesResourceWithStreamingResponse:
        return AsyncCivicAddressesResourceWithStreamingResponse(self._external_connections.civic_addresses)

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        return AsyncPhoneNumbersResourceWithStreamingResponse(self._external_connections.phone_numbers)

    @cached_property
    def releases(self) -> AsyncReleasesResourceWithStreamingResponse:
        return AsyncReleasesResourceWithStreamingResponse(self._external_connections.releases)

    @cached_property
    def uploads(self) -> AsyncUploadsResourceWithStreamingResponse:
        return AsyncUploadsResourceWithStreamingResponse(self._external_connections.uploads)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .verify_meta import VerifyMeta as VerifyMeta
from .action_verify_params import ActionVerifyParams as ActionVerifyParams
from .by_phone_number_list_response import ByPhoneNumberListResponse as ByPhoneNumberListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["MessagingHostedNumberOrderValidateCodesResponse", "Data", "DataPhoneNumber"]


class DataPhoneNumber(BaseModel):
    phone_number: str

    status: Literal["verified", "rejected", "already_verified"]


class Data(BaseModel):
    order_id: str

    phone_numbers: List[DataPhoneNumber]


class MessagingHostedNumberOrderValidateCodesResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.texml.accounts.conferences import (
    ParticipantUpdateResponse,
    ParticipantRetrieveResponse,
    ParticipantParticipantsResponse,
    ParticipantRetrieveParticipantsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestParticipants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.retrieve(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )
        assert_matches_type(ParticipantRetrieveResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.texml.accounts.conferences.participants.with_raw_response.retrieve(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = response.parse()
        assert_matches_type(ParticipantRetrieveResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.texml.accounts.conferences.participants.with_streaming_response.retrieve(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = response.parse()
            assert_matches_type(ParticipantRetrieveResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.retrieve(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="",
                conference_sid="conference_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.retrieve(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="account_sid",
                conference_sid="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `call_sid_or_participant_label` but received ''"
        ):
            client.texml.accounts.conferences.participants.with_raw_response.retrieve(
                call_sid_or_participant_label="",
                account_sid="account_sid",
                conference_sid="conference_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )
        assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
            announce_method="GET",
            announce_url="https://www.example.com/announce.xml",
            beep_on_exit=False,
            call_sid_to_coach="v3:9X2vxPDFY2RHSJ1EdMS0RHRksMTg7ldNxdjWbVr9zBjbGjGsSe-aiQ",
            coaching=False,
            end_conference_on_exit=False,
            hold=True,
            hold_method="POST",
            hold_url="https://www.example.com/hold-music.xml",
            muted=True,
            wait_url="https://www.example.com/wait_music.mp3",
        )
        assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.texml.accounts.conferences.participants.with_raw_response.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = response.parse()
        assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.texml.accounts.conferences.participants.with_streaming_response.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = response.parse()
            assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.update(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="",
                conference_sid="conference_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.update(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="account_sid",
                conference_sid="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `call_sid_or_participant_label` but received ''"
        ):
            client.texml.accounts.conferences.participants.with_raw_response.update(
                call_sid_or_participant_label="",
                account_sid="account_sid",
                conference_sid="conference_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.delete(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )
        assert participant is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.texml.accounts.conferences.participants.with_raw_response.delete(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = response.parse()
        assert participant is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.texml.accounts.conferences.participants.with_streaming_response.delete(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = response.parse()
            assert participant is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.delete(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="",
                conference_sid="conference_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.delete(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="account_sid",
                conference_sid="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `call_sid_or_participant_label` but received ''"
        ):
            client.texml.accounts.conferences.participants.with_raw_response.delete(
                call_sid_or_participant_label="",
                account_sid="account_sid",
                conference_sid="conference_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_participants(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )
        assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_participants_with_all_params(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
            amd_status_callback="https://www.example.com/amd_result",
            amd_status_callback_method="GET",
            beep="onExit",
            caller_id="Info",
            call_sid_to_coach="v3:9X2vxPDFY2RHSJ1EdMS0RHRksMTg7ldNxdjWbVr9zBjbGjGsSe-aiQ",
            cancel_playback_on_detect_message_end=False,
            cancel_playback_on_machine_detection=False,
            coaching=False,
            conference_record="record-from-start",
            conference_recording_status_callback="https://example.com/conference_recording_status_callback",
            conference_recording_status_callback_event="in-progress completed failed absent",
            conference_recording_status_callback_method="GET",
            conference_recording_timeout=5,
            conference_status_callback="https://example.com/conference_status_callback",
            conference_status_callback_event="start end join leave",
            conference_status_callback_method="GET",
            conference_trim="trim-silence",
            custom_headers=[
                {
                    "name": "X-Custom-Header",
                    "value": "custom-value",
                }
            ],
            early_media=True,
            end_conference_on_exit=True,
            from_="+12065550200",
            machine_detection="Enable",
            machine_detection_silence_timeout=2000,
            machine_detection_speech_end_threshold=2000,
            machine_detection_speech_threshold=2000,
            machine_detection_timeout=1000,
            max_participants=30,
            muted=True,
            preferred_codecs="PCMA,PCMU",
            record=False,
            recording_channels="dual",
            recording_status_callback="https://example.com/recording_status_callback",
            recording_status_callback_event="in-progress completed absent",
            recording_status_callback_method="GET",
            recording_track="inbound",
            sip_auth_password="1234",
            sip_auth_username="user",
            start_conference_on_enter=False,
            status_callback="https://www.example.com/callback",
            status_callback_event="answered completed",
            status_callback_method="GET",
            time_limit=30,
            timeout_seconds=30,
            to="+12065550100",
            trim="trim-silence",
            wait_url="https://www.example.com/wait_music.mp3",
        )
        assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_participants(self, client: Telnyx) -> None:
        response = client.texml.accounts.conferences.participants.with_raw_response.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = response.parse()
        assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_participants(self, client: Telnyx) -> None:
        with client.texml.accounts.conferences.participants.with_streaming_response.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = response.parse()
            assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_participants(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.participants(
                conference_sid="conference_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.participants(
                conference_sid="",
                account_sid="account_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_participants(self, client: Telnyx) -> None:
        participant = client.texml.accounts.conferences.participants.retrieve_participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )
        assert_matches_type(ParticipantRetrieveParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_participants(self, client: Telnyx) -> None:
        response = client.texml.accounts.conferences.participants.with_raw_response.retrieve_participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = response.parse()
        assert_matches_type(ParticipantRetrieveParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_participants(self, client: Telnyx) -> None:
        with client.texml.accounts.conferences.participants.with_streaming_response.retrieve_participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = response.parse()
            assert_matches_type(ParticipantRetrieveParticipantsResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_participants(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.retrieve_participants(
                conference_sid="conference_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            client.texml.accounts.conferences.participants.with_raw_response.retrieve_participants(
                conference_sid="",
                account_sid="account_sid",
            )


class TestAsyncParticipants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.retrieve(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )
        assert_matches_type(ParticipantRetrieveResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = await response.parse()
        assert_matches_type(ParticipantRetrieveResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.conferences.participants.with_streaming_response.retrieve(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = await response.parse()
            assert_matches_type(ParticipantRetrieveResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="",
                conference_sid="conference_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="account_sid",
                conference_sid="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `call_sid_or_participant_label` but received ''"
        ):
            await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve(
                call_sid_or_participant_label="",
                account_sid="account_sid",
                conference_sid="conference_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )
        assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
            announce_method="GET",
            announce_url="https://www.example.com/announce.xml",
            beep_on_exit=False,
            call_sid_to_coach="v3:9X2vxPDFY2RHSJ1EdMS0RHRksMTg7ldNxdjWbVr9zBjbGjGsSe-aiQ",
            coaching=False,
            end_conference_on_exit=False,
            hold=True,
            hold_method="POST",
            hold_url="https://www.example.com/hold-music.xml",
            muted=True,
            wait_url="https://www.example.com/wait_music.mp3",
        )
        assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.conferences.participants.with_raw_response.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = await response.parse()
        assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.conferences.participants.with_streaming_response.update(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = await response.parse()
            assert_matches_type(ParticipantUpdateResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.update(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="",
                conference_sid="conference_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.update(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="account_sid",
                conference_sid="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `call_sid_or_participant_label` but received ''"
        ):
            await async_client.texml.accounts.conferences.participants.with_raw_response.update(
                call_sid_or_participant_label="",
                account_sid="account_sid",
                conference_sid="conference_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.delete(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )
        assert participant is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.conferences.participants.with_raw_response.delete(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = await response.parse()
        assert participant is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.conferences.participants.with_streaming_response.delete(
            call_sid_or_participant_label="call_sid_or_participant_label",
            account_sid="account_sid",
            conference_sid="conference_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = await response.parse()
            assert participant is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.delete(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="",
                conference_sid="conference_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.delete(
                call_sid_or_participant_label="call_sid_or_participant_label",
                account_sid="account_sid",
                conference_sid="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `call_sid_or_participant_label` but received ''"
        ):
            await async_client.texml.accounts.conferences.participants.with_raw_response.delete(
                call_sid_or_participant_label="",
                account_sid="account_sid",
                conference_sid="conference_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_participants(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )
        assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_participants_with_all_params(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
            amd_status_callback="https://www.example.com/amd_result",
            amd_status_callback_method="GET",
            beep="onExit",
            caller_id="Info",
            call_sid_to_coach="v3:9X2vxPDFY2RHSJ1EdMS0RHRksMTg7ldNxdjWbVr9zBjbGjGsSe-aiQ",
            cancel_playback_on_detect_message_end=False,
            cancel_playback_on_machine_detection=False,
            coaching=False,
            conference_record="record-from-start",
            conference_recording_status_callback="https://example.com/conference_recording_status_callback",
            conference_recording_status_callback_event="in-progress completed failed absent",
            conference_recording_status_callback_method="GET",
            conference_recording_timeout=5,
            conference_status_callback="https://example.com/conference_status_callback",
            conference_status_callback_event="start end join leave",
            conference_status_callback_method="GET",
            conference_trim="trim-silence",
            custom_headers=[
                {
                    "name": "X-Custom-Header",
                    "value": "custom-value",
                }
            ],
            early_media=True,
            end_conference_on_exit=True,
            from_="+12065550200",
            machine_detection="Enable",
            machine_detection_silence_timeout=2000,
            machine_detection_speech_end_threshold=2000,
            machine_detection_speech_threshold=2000,
            machine_detection_timeout=1000,
            max_participants=30,
            muted=True,
            preferred_codecs="PCMA,PCMU",
            record=False,
            recording_channels="dual",
            recording_status_callback="https://example.com/recording_status_callback",
            recording_status_callback_event="in-progress completed absent",
            recording_status_callback_method="GET",
            recording_track="inbound",
            sip_auth_password="1234",
            sip_auth_username="user",
            start_conference_on_enter=False,
            status_callback="https://www.example.com/callback",
            status_callback_event="answered completed",
            status_callback_method="GET",
            time_limit=30,
            timeout_seconds=30,
            to="+12065550100",
            trim="trim-silence",
            wait_url="https://www.example.com/wait_music.mp3",
        )
        assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_participants(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.conferences.participants.with_raw_response.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = await response.parse()
        assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_participants(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.conferences.participants.with_streaming_response.participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = await response.parse()
            assert_matches_type(ParticipantParticipantsResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_participants(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.participants(
                conference_sid="conference_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.participants(
                conference_sid="",
                account_sid="account_sid",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        participant = await async_client.texml.accounts.conferences.participants.retrieve_participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )
        assert_matches_type(ParticipantRetrieveParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve_participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        participant = await response.parse()
        assert_matches_type(ParticipantRetrieveParticipantsResponse, participant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.conferences.participants.with_streaming_response.retrieve_participants(
            conference_sid="conference_sid",
            account_sid="account_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            participant = await response.parse()
            assert_matches_type(ParticipantRetrieveParticipantsResponse, participant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_participants(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve_participants(
                conference_sid="conference_sid",
                account_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conference_sid` but received ''"):
            await async_client.texml.accounts.conferences.participants.with_raw_response.retrieve_participants(
                conference_sid="",
                account_sid="account_sid",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_hangup import CallHangup

__all__ = ["CallHangupWebhookEvent"]


class CallHangupWebhookEvent(BaseModel):
    data: Optional[CallHangup] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .fax import Fax
from .._models import BaseModel

__all__ = ["FaxRetrieveResponse"]


class FaxRetrieveResponse(BaseModel):
    data: Optional[Fax] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .whatsapp_media import WhatsappMedia

__all__ = [
    "MessageSendWhatsappResponse",
    "Data",
    "DataBody",
    "DataBodyContact",
    "DataBodyContactAddress",
    "DataBodyContactEmail",
    "DataBodyContactOrg",
    "DataBodyContactPhone",
    "DataBodyContactURL",
    "DataBodyInteractive",
    "DataBodyInteractiveAction",
    "DataBodyInteractiveActionButton",
    "DataBodyInteractiveActionButtonReply",
    "DataBodyInteractiveActionCard",
    "DataBodyInteractiveActionCardAction",
    "DataBodyInteractiveActionCardBody",
    "DataBodyInteractiveActionCardHeader",
    "DataBodyInteractiveActionParameters",
    "DataBodyInteractiveActionSection",
    "DataBodyInteractiveActionSectionProductItem",
    "DataBodyInteractiveActionSectionRow",
    "DataBodyInteractiveBody",
    "DataBodyInteractiveFooter",
    "DataBodyInteractiveHeader",
    "DataBodyLocation",
    "DataBodyReaction",
    "DataFrom",
    "DataTo",
]


class DataBodyContactAddress(BaseModel):
    city: Optional[str] = None

    country: Optional[str] = None

    country_code: Optional[str] = None

    state: Optional[str] = None

    street: Optional[str] = None

    type: Optional[str] = None

    zip: Optional[str] = None


class DataBodyContactEmail(BaseModel):
    email: Optional[str] = None

    type: Optional[str] = None


class DataBodyContactOrg(BaseModel):
    company: Optional[str] = None

    department: Optional[str] = None

    title: Optional[str] = None


class DataBodyContactPhone(BaseModel):
    phone: Optional[str] = None

    type: Optional[str] = None

    wa_id: Optional[str] = None


class DataBodyContactURL(BaseModel):
    type: Optional[str] = None

    url: Optional[str] = None


class DataBodyContact(BaseModel):
    addresses: Optional[List[DataBodyContactAddress]] = None

    birthday: Optional[str] = None

    emails: Optional[List[DataBodyContactEmail]] = None

    name: Optional[str] = None

    org: Optional[DataBodyContactOrg] = None

    phones: Optional[List[DataBodyContactPhone]] = None

    urls: Optional[List[DataBodyContactURL]] = None


class DataBodyInteractiveActionButtonReply(BaseModel):
    id: Optional[str] = None
    """unique identifier for each button, 256 character maximum"""

    title: Optional[str] = None
    """button label, 20 character maximum"""


class DataBodyInteractiveActionButton(BaseModel):
    reply: Optional[DataBodyInteractiveActionButtonReply] = None

    type: Optional[Literal["reply"]] = None


class DataBodyInteractiveActionCardAction(BaseModel):
    catalog_id: Optional[str] = None
    """the unique ID of the catalog"""

    product_retailer_id: Optional[str] = None
    """the unique retailer ID of the product"""


class DataBodyInteractiveActionCardBody(BaseModel):
    text: Optional[str] = None
    """160 character maximum, up to 2 line breaks"""


class DataBodyInteractiveActionCardHeader(BaseModel):
    image: Optional[WhatsappMedia] = None

    type: Optional[Literal["image", "video"]] = None

    video: Optional[WhatsappMedia] = None


class DataBodyInteractiveActionCard(BaseModel):
    action: Optional[DataBodyInteractiveActionCardAction] = None

    body: Optional[DataBodyInteractiveActionCardBody] = None

    card_index: Optional[int] = None
    """unique index for each card (0-9)"""

    header: Optional[DataBodyInteractiveActionCardHeader] = None

    type: Optional[Literal["cta_url"]] = None


class DataBodyInteractiveActionParameters(BaseModel):
    display_text: Optional[str] = None
    """button label text, 20 character maximum"""

    url: Optional[str] = None
    """button URL to load when tapped by the user"""


class DataBodyInteractiveActionSectionProductItem(BaseModel):
    product_retailer_id: Optional[str] = None


class DataBodyInteractiveActionSectionRow(BaseModel):
    id: Optional[str] = None
    """arbitrary string identifying the row, 200 character maximum"""

    description: Optional[str] = None
    """row description, 72 character maximum"""

    title: Optional[str] = None
    """row title, 24 character maximum"""


class DataBodyInteractiveActionSection(BaseModel):
    product_items: Optional[List[DataBodyInteractiveActionSectionProductItem]] = None

    rows: Optional[List[DataBodyInteractiveActionSectionRow]] = None

    title: Optional[str] = None
    """section title, 24 character maximum"""


class DataBodyInteractiveAction(BaseModel):
    button: Optional[str] = None

    buttons: Optional[List[DataBodyInteractiveActionButton]] = None

    cards: Optional[List[DataBodyInteractiveActionCard]] = None

    catalog_id: Optional[str] = None

    mode: Optional[str] = None

    name: Optional[str] = None

    parameters: Optional[DataBodyInteractiveActionParameters] = None

    product_retailer_id: Optional[str] = None

    sections: Optional[List[DataBodyInteractiveActionSection]] = None


class DataBodyInteractiveBody(BaseModel):
    text: Optional[str] = None
    """body text, 1024 character maximum"""


class DataBodyInteractiveFooter(BaseModel):
    text: Optional[str] = None
    """footer text, 60 character maximum"""


class DataBodyInteractiveHeader(BaseModel):
    document: Optional[WhatsappMedia] = None

    image: Optional[WhatsappMedia] = None

    sub_text: Optional[str] = None

    text: Optional[str] = None
    """header text, 60 character maximum"""

    video: Optional[WhatsappMedia] = None


class DataBodyInteractive(BaseModel):
    action: Optional[DataBodyInteractiveAction] = None

    body: Optional[DataBodyInteractiveBody] = None

    footer: Optional[DataBodyInteractiveFooter] = None

    header: Optional[DataBodyInteractiveHeader] = None

    type: Optional[Literal["cta_url", "list", "carousel", "button", "location_request_message"]] = None


class DataBodyLocation(BaseModel):
    address: Optional[str] = None

    latitude: Optional[str] = None

    longitude: Optional[str] = None

    name: Optional[str] = None


class DataBodyReaction(BaseModel):
    emoji: Optional[str] = None

    message_id: Optional[str] = None


class DataBody(BaseModel):
    audio: Optional[WhatsappMedia] = None

    biz_opaque_callback_data: Optional[str] = None
    """custom data to return with status update"""

    contacts: Optional[List[DataBodyContact]] = None

    document: Optional[WhatsappMedia] = None

    image: Optional[WhatsappMedia] = None

    interactive: Optional[DataBodyInteractive] = None

    location: Optional[DataBodyLocation] = None

    reaction: Optional[DataBodyReaction] = None

    sticker: Optional[WhatsappMedia] = None

    type: Optional[
        Literal[
            "audio",
            "document",
            "image",
            "sticker",
            "video",
            "interactive",
            "location",
            "template",
            "reaction",
            "contacts",
        ]
    ] = None

    video: Optional[WhatsappMedia] = None


class DataFrom(BaseModel):
    carrier: Optional[str] = None
    """The carrier of the sender."""

    line_type: Optional[Literal["Wireline", "Wireless", "VoWiFi", "VoIP", "Pre-Paid Wireless", ""]] = None
    """The line-type of the sender."""

    phone_number: Optional[str] = None
    """
    Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
    code).
    """

    status: Optional[Literal["received", "delivered"]] = None


class DataTo(BaseModel):
    carrier: Optional[str] = None

    line_type: Optional[str] = None

    phone_number: Optional[str] = None

    status: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None
    """message ID"""

    body: Optional[DataBody] = None

    direction: Optional[str] = None

    encoding: Optional[str] = None

    from_: Optional[DataFrom] = FieldInfo(alias="from", default=None)

    messaging_profile_id: Optional[str] = None

    organization_id: Optional[str] = None

    received_at: Optional[datetime] = None

    record_type: Optional[str] = None

    to: Optional[List[DataTo]] = None

    type: Optional[str] = None


class MessageSendWhatsappResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionUnmuteParams"]


class ActionUnmuteParams(TypedDict, total=False):
    call_control_ids: SequenceNotStr[str]
    """List of unique identifiers and tokens for controlling the call.

    Enter each call control ID to be unmuted. When empty all participants will be
    unmuted.
    """

    region: Literal["Australia", "Europe", "Middle East", "US"]
    """Region where the conference data is located.

    Defaults to the region defined in user's data locality settings (Europe or US).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_bridged import CallBridged

__all__ = ["CallBridgedWebhookEvent"]


class CallBridgedWebhookEvent(BaseModel):
    data: Optional[CallBridged] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ....types.rooms import (
    session_list_0_params,
    session_list_1_params,
    session_retrieve_params,
    session_retrieve_participants_params,
)
from ...._base_client import AsyncPaginator, make_request_options
from ....types.room_session import RoomSession
from ....types.shared.room_participant import RoomParticipant
from ....types.rooms.session_retrieve_response import SessionRetrieveResponse

__all__ = ["SessionsResource", "AsyncSessionsResource"]


class SessionsResource(SyncAPIResource):
    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SessionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        room_session_id: str,
        *,
        include_participants: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionRetrieveResponse:
        """
        View a room session.

        Args:
          include_participants: To decide if room participants should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._get(
            f"/room_sessions/{room_session_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include_participants": include_participants}, session_retrieve_params.SessionRetrieveParams
                ),
            ),
            cast_to=SessionRetrieveResponse,
        )

    def list_0(
        self,
        *,
        filter: session_list_0_params.Filter | Omit = omit,
        include_participants: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[RoomSession]:
        """
        View a list of room sessions.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte],
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[room_id], filter[active]

          include_participants: To decide if room participants should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/room_sessions",
            page=SyncDefaultFlatPagination[RoomSession],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_participants": include_participants,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    session_list_0_params.SessionList0Params,
                ),
            ),
            model=RoomSession,
        )

    def list_1(
        self,
        room_id: str,
        *,
        filter: session_list_1_params.Filter | Omit = omit,
        include_participants: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[RoomSession]:
        """
        View a list of room sessions.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte],
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[active]

          include_participants: To decide if room participants should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return self._get_api_list(
            f"/rooms/{room_id}/sessions",
            page=SyncDefaultFlatPagination[RoomSession],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_participants": include_participants,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    session_list_1_params.SessionList1Params,
                ),
            ),
            model=RoomSession,
        )

    def retrieve_participants(
        self,
        room_session_id: str,
        *,
        filter: session_retrieve_participants_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[RoomParticipant]:
        """
        View a list of room participants.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_joined_at][eq], filter[date_joined_at][gte],
              filter[date_joined_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte],
              filter[date_left_at][eq], filter[date_left_at][gte], filter[date_left_at][lte],
              filter[context]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._get_api_list(
            f"/room_sessions/{room_session_id}/participants",
            page=SyncDefaultFlatPagination[RoomParticipant],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    session_retrieve_participants_params.SessionRetrieveParticipantsParams,
                ),
            ),
            model=RoomParticipant,
        )


class AsyncSessionsResource(AsyncAPIResource):
    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSessionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        room_session_id: str,
        *,
        include_participants: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionRetrieveResponse:
        """
        View a room session.

        Args:
          include_participants: To decide if room participants should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return await self._get(
            f"/room_sessions/{room_session_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_participants": include_participants}, session_retrieve_params.SessionRetrieveParams
                ),
            ),
            cast_to=SessionRetrieveResponse,
        )

    def list_0(
        self,
        *,
        filter: session_list_0_params.Filter | Omit = omit,
        include_participants: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RoomSession, AsyncDefaultFlatPagination[RoomSession]]:
        """
        View a list of room sessions.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte],
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[room_id], filter[active]

          include_participants: To decide if room participants should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/room_sessions",
            page=AsyncDefaultFlatPagination[RoomSession],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_participants": include_participants,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    session_list_0_params.SessionList0Params,
                ),
            ),
            model=RoomSession,
        )

    def list_1(
        self,
        room_id: str,
        *,
        filter: session_list_1_params.Filter | Omit = omit,
        include_participants: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RoomSession, AsyncDefaultFlatPagination[RoomSession]]:
        """
        View a list of room sessions.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_created_at][eq], filter[date_created_at][gte],
              filter[date_created_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte],
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[active]

          include_participants: To decide if room participants should be included in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return self._get_api_list(
            f"/rooms/{room_id}/sessions",
            page=AsyncDefaultFlatPagination[RoomSession],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "include_participants": include_participants,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    session_list_1_params.SessionList1Params,
                ),
            ),
            model=RoomSession,
        )

    def retrieve_participants(
        self,
        room_session_id: str,
        *,
        filter: session_retrieve_participants_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RoomParticipant, AsyncDefaultFlatPagination[RoomParticipant]]:
        """
        View a list of room participants.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_joined_at][eq], filter[date_joined_at][gte],
              filter[date_joined_at][lte], filter[date_updated_at][eq],
              filter[date_updated_at][gte], filter[date_updated_at][lte],
              filter[date_left_at][eq], filter[date_left_at][gte], filter[date_left_at][lte],
              filter[context]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._get_api_list(
            f"/room_sessions/{room_session_id}/participants",
            page=AsyncDefaultFlatPagination[RoomParticipant],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    session_retrieve_participants_params.SessionRetrieveParticipantsParams,
                ),
            ),
            model=RoomParticipant,
        )


class SessionsResourceWithRawResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.retrieve = to_raw_response_wrapper(
            sessions.retrieve,
        )
        self.list_0 = to_raw_response_wrapper(
            sessions.list_0,
        )
        self.list_1 = to_raw_response_wrapper(
            sessions.list_1,
        )
        self.retrieve_participants = to_raw_response_wrapper(
            sessions.retrieve_participants,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._sessions.actions)


class AsyncSessionsResourceWithRawResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.retrieve = async_to_raw_response_wrapper(
            sessions.retrieve,
        )
        self.list_0 = async_to_raw_response_wrapper(
            sessions.list_0,
        )
        self.list_1 = async_to_raw_response_wrapper(
            sessions.list_1,
        )
        self.retrieve_participants = async_to_raw_response_wrapper(
            sessions.retrieve_participants,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._sessions.actions)


class SessionsResourceWithStreamingResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.retrieve = to_streamed_response_wrapper(
            sessions.retrieve,
        )
        self.list_0 = to_streamed_response_wrapper(
            sessions.list_0,
        )
        self.list_1 = to_streamed_response_wrapper(
            sessions.list_1,
        )
        self.retrieve_participants = to_streamed_response_wrapper(
            sessions.retrieve_participants,
        )

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._sessions.actions)


class AsyncSessionsResourceWithStreamingResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.retrieve = async_to_streamed_response_wrapper(
            sessions.retrieve,
        )
        self.list_0 = async_to_streamed_response_wrapper(
            sessions.list_0,
        )
        self.list_1 = async_to_streamed_response_wrapper(
            sessions.list_1,
        )
        self.retrieve_participants = async_to_streamed_response_wrapper(
            sessions.retrieve_participants,
        )

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._sessions.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["SttDetailReportResponse"]


class SttDetailReportResponse(BaseModel):
    id: Optional[str] = None
    """Identifies the resource"""

    created_at: Optional[datetime] = None

    download_link: Optional[str] = None
    """URL to download the report"""

    end_date: Optional[datetime] = None

    record_type: Optional[str] = None

    start_date: Optional[datetime] = None

    status: Optional[Literal["PENDING", "COMPLETE", "FAILED", "EXPIRED"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .messaging_list_params import MessagingListParams as MessagingListParams
from .messaging_list_response import MessagingListResponse as MessagingListResponse
from .messaging_retrieve_response import MessagingRetrieveResponse as MessagingRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Union, cast
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.ai.assistants import ConversationChannelType, scheduled_event_list_params, scheduled_event_create_params
from ....types.ai.assistants.scheduled_event_response import ScheduledEventResponse
from ....types.ai.assistants.conversation_channel_type import ConversationChannelType
from ....types.ai.assistants.scheduled_event_list_response import ScheduledEventListResponse

__all__ = ["ScheduledEventsResource", "AsyncScheduledEventsResource"]


class ScheduledEventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ScheduledEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ScheduledEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScheduledEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ScheduledEventsResourceWithStreamingResponse(self)

    def create(
        self,
        assistant_id: str,
        *,
        scheduled_at_fixed_datetime: Union[str, datetime],
        telnyx_agent_target: str,
        telnyx_conversation_channel: ConversationChannelType,
        telnyx_end_user_target: str,
        conversation_metadata: Dict[str, Union[str, int, bool]] | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEventResponse:
        """
        Create a scheduled event for an assistant

        Args:
          scheduled_at_fixed_datetime: The datetime at which the event should be scheduled. Formatted as ISO 8601.

          telnyx_agent_target: The phone number, SIP URI, to schedule the call or text from.

          telnyx_end_user_target: The phone number, SIP URI, to schedule the call or text to.

          conversation_metadata: Metadata associated with the conversation. Telnyx provides several pieces of
              metadata, but customers can also add their own.

          text: Required for sms scheduled events. The text to be sent to the end user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return cast(
            ScheduledEventResponse,
            self._post(
                f"/ai/assistants/{assistant_id}/scheduled_events",
                body=maybe_transform(
                    {
                        "scheduled_at_fixed_datetime": scheduled_at_fixed_datetime,
                        "telnyx_agent_target": telnyx_agent_target,
                        "telnyx_conversation_channel": telnyx_conversation_channel,
                        "telnyx_end_user_target": telnyx_end_user_target,
                        "conversation_metadata": conversation_metadata,
                        "text": text,
                    },
                    scheduled_event_create_params.ScheduledEventCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduledEventResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        event_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEventResponse:
        """
        Retrieve a scheduled event by event ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return cast(
            ScheduledEventResponse,
            self._get(
                f"/ai/assistants/{assistant_id}/scheduled_events/{event_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduledEventResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        assistant_id: str,
        *,
        conversation_channel: ConversationChannelType | Omit = omit,
        from_date: Union[str, datetime] | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        to_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ScheduledEventListResponse]:
        """
        Get scheduled events for an assistant with pagination and filtering

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._get_api_list(
            f"/ai/assistants/{assistant_id}/scheduled_events",
            page=SyncDefaultFlatPagination[ScheduledEventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "conversation_channel": conversation_channel,
                        "from_date": from_date,
                        "page_number": page_number,
                        "page_size": page_size,
                        "to_date": to_date,
                    },
                    scheduled_event_list_params.ScheduledEventListParams,
                ),
            ),
            model=cast(
                Any, ScheduledEventListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        event_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """If the event is pending, this will cancel the event.

        Otherwise, this will simply
        remove the record of the event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/assistants/{assistant_id}/scheduled_events/{event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncScheduledEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncScheduledEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScheduledEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScheduledEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncScheduledEventsResourceWithStreamingResponse(self)

    async def create(
        self,
        assistant_id: str,
        *,
        scheduled_at_fixed_datetime: Union[str, datetime],
        telnyx_agent_target: str,
        telnyx_conversation_channel: ConversationChannelType,
        telnyx_end_user_target: str,
        conversation_metadata: Dict[str, Union[str, int, bool]] | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEventResponse:
        """
        Create a scheduled event for an assistant

        Args:
          scheduled_at_fixed_datetime: The datetime at which the event should be scheduled. Formatted as ISO 8601.

          telnyx_agent_target: The phone number, SIP URI, to schedule the call or text from.

          telnyx_end_user_target: The phone number, SIP URI, to schedule the call or text to.

          conversation_metadata: Metadata associated with the conversation. Telnyx provides several pieces of
              metadata, but customers can also add their own.

          text: Required for sms scheduled events. The text to be sent to the end user.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return cast(
            ScheduledEventResponse,
            await self._post(
                f"/ai/assistants/{assistant_id}/scheduled_events",
                body=await async_maybe_transform(
                    {
                        "scheduled_at_fixed_datetime": scheduled_at_fixed_datetime,
                        "telnyx_agent_target": telnyx_agent_target,
                        "telnyx_conversation_channel": telnyx_conversation_channel,
                        "telnyx_end_user_target": telnyx_end_user_target,
                        "conversation_metadata": conversation_metadata,
                        "text": text,
                    },
                    scheduled_event_create_params.ScheduledEventCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduledEventResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        event_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ScheduledEventResponse:
        """
        Retrieve a scheduled event by event ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return cast(
            ScheduledEventResponse,
            await self._get(
                f"/ai/assistants/{assistant_id}/scheduled_events/{event_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ScheduledEventResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        assistant_id: str,
        *,
        conversation_channel: ConversationChannelType | Omit = omit,
        from_date: Union[str, datetime] | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        to_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ScheduledEventListResponse, AsyncDefaultFlatPagination[ScheduledEventListResponse]]:
        """
        Get scheduled events for an assistant with pagination and filtering

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._get_api_list(
            f"/ai/assistants/{assistant_id}/scheduled_events",
            page=AsyncDefaultFlatPagination[ScheduledEventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "conversation_channel": conversation_channel,
                        "from_date": from_date,
                        "page_number": page_number,
                        "page_size": page_size,
                        "to_date": to_date,
                    },
                    scheduled_event_list_params.ScheduledEventListParams,
                ),
            ),
            model=cast(
                Any, ScheduledEventListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        event_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """If the event is pending, this will cancel the event.

        Otherwise, this will simply
        remove the record of the event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/assistants/{assistant_id}/scheduled_events/{event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ScheduledEventsResourceWithRawResponse:
    def __init__(self, scheduled_events: ScheduledEventsResource) -> None:
        self._scheduled_events = scheduled_events

        self.create = to_raw_response_wrapper(
            scheduled_events.create,
        )
        self.retrieve = to_raw_response_wrapper(
            scheduled_events.retrieve,
        )
        self.list = to_raw_response_wrapper(
            scheduled_events.list,
        )
        self.delete = to_raw_response_wrapper(
            scheduled_events.delete,
        )


class AsyncScheduledEventsResourceWithRawResponse:
    def __init__(self, scheduled_events: AsyncScheduledEventsResource) -> None:
        self._scheduled_events = scheduled_events

        self.create = async_to_raw_response_wrapper(
            scheduled_events.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            scheduled_events.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            scheduled_events.list,
        )
        self.delete = async_to_raw_response_wrapper(
            scheduled_events.delete,
        )


class ScheduledEventsResourceWithStreamingResponse:
    def __init__(self, scheduled_events: ScheduledEventsResource) -> None:
        self._scheduled_events = scheduled_events

        self.create = to_streamed_response_wrapper(
            scheduled_events.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            scheduled_events.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            scheduled_events.list,
        )
        self.delete = to_streamed_response_wrapper(
            scheduled_events.delete,
        )


class AsyncScheduledEventsResourceWithStreamingResponse:
    def __init__(self, scheduled_events: AsyncScheduledEventsResource) -> None:
        self._scheduled_events = scheduled_events

        self.create = async_to_streamed_response_wrapper(
            scheduled_events.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            scheduled_events.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            scheduled_events.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            scheduled_events.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "EventRetrieveResponse",
    "Data",
    "DataWebhookPortoutStatusChanged",
    "DataWebhookPortoutStatusChangedPayload",
    "DataWebhookPortoutNewComment",
    "DataWebhookPortoutNewCommentPayload",
    "DataWebhookPortoutFocDateChanged",
    "DataWebhookPortoutFocDateChangedPayload",
]


class DataWebhookPortoutStatusChangedPayload(BaseModel):
    """The webhook payload for the portout.status_changed event"""

    id: Optional[str] = None
    """Identifies the port out that was moved."""

    attempted_pin: Optional[str] = None
    """The PIN that was attempted to be used to authorize the port out."""

    carrier_name: Optional[str] = None
    """Carrier the number will be ported out to"""

    phone_numbers: Optional[List[str]] = None
    """Phone numbers associated with this port-out order"""

    rejection_reason: Optional[str] = None
    """The reason why the order is being rejected by the user.

    If the order is authorized, this field can be left null
    """

    spid: Optional[str] = None
    """The new carrier SPID."""

    status: Optional[Literal["pending", "authorized", "ported", "rejected", "rejected-pending", "canceled"]] = None
    """The new status of the port out."""

    subscriber_name: Optional[str] = None
    """The name of the port-out's end user."""

    user_id: Optional[str] = None
    """Identifies the user that the port-out order belongs to."""


class DataWebhookPortoutStatusChanged(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[Literal["portout.status_changed", "portout.foc_date_changed", "portout.new_comment"]] = None
    """Identifies the event type"""

    payload: Optional[DataWebhookPortoutStatusChangedPayload] = None
    """The webhook payload for the portout.status_changed event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    portout_id: Optional[str] = None
    """Identifies the port-out order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class DataWebhookPortoutNewCommentPayload(BaseModel):
    """The webhook payload for the portout.new_comment event"""

    id: Optional[str] = None
    """Identifies the comment that was added to the port-out order."""

    comment: Optional[str] = None
    """The body of the comment."""

    portout_id: Optional[str] = None
    """Identifies the port-out order that the comment was added to."""

    user_id: Optional[str] = None
    """Identifies the user that added the comment."""


class DataWebhookPortoutNewComment(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[Literal["portout.status_changed", "portout.foc_date_changed", "portout.new_comment"]] = None
    """Identifies the event type"""

    payload: Optional[DataWebhookPortoutNewCommentPayload] = None
    """The webhook payload for the portout.new_comment event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    portout_id: Optional[str] = None
    """Identifies the port-out order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class DataWebhookPortoutFocDateChangedPayload(BaseModel):
    """The webhook payload for the portout.foc_date_changed event"""

    id: Optional[str] = None
    """Identifies the port-out order that have the FOC date changed."""

    foc_date: Optional[datetime] = None
    """ISO 8601 formatted date indicating the new FOC date."""

    user_id: Optional[str] = None
    """Identifies the organization that port-out order belongs to."""


class DataWebhookPortoutFocDateChanged(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[Literal["portout.status_changed", "portout.foc_date_changed", "portout.new_comment"]] = None
    """Identifies the event type"""

    payload: Optional[DataWebhookPortoutFocDateChangedPayload] = None
    """The webhook payload for the portout.foc_date_changed event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    portout_id: Optional[str] = None
    """Identifies the port-out order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


Data: TypeAlias = Union[DataWebhookPortoutStatusChanged, DataWebhookPortoutNewComment, DataWebhookPortoutFocDateChanged]


class EventRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["CallSiprecJsonResponse"]


class CallSiprecJsonResponse(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    call_sid: Optional[str] = None
    """The id of the call the resource belongs to."""

    date_created: Optional[str] = None
    """The date and time the siprec session was created."""

    date_updated: Optional[str] = None
    """The date and time the siprec session was last updated."""

    error_code: Optional[str] = None
    """The error code of the siprec session."""

    sid: Optional[str] = None
    """The SID of the siprec session."""

    start_time: Optional[str] = None
    """The date and time the siprec session was started."""

    status: Optional[Literal["in-progress", "stopped"]] = None
    """The status of the siprec session."""

    track: Optional[Literal["both_tracks", "inbound_track", "outbound_track"]] = None
    """The track used for the siprec session."""

    uri: Optional[str] = None
    """The URI of the siprec session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .ledger_billing_group_report import LedgerBillingGroupReport

__all__ = ["LedgerBillingGroupReportRetrieveResponse"]


class LedgerBillingGroupReportRetrieveResponse(BaseModel):
    data: Optional[LedgerBillingGroupReport] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    AuthenticationProvider,
    AuthenticationProviderCreateResponse,
    AuthenticationProviderDeleteResponse,
    AuthenticationProviderUpdateResponse,
    AuthenticationProviderRetrieveResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuthenticationProviders:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
            },
            short_name="myorg",
        )
        assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
                "idp_cert_fingerprint_algorithm": "sha256",
            },
            short_name="myorg",
            active=True,
            settings_url="https://myorg.myidp.com/saml/metadata",
        )
        assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.authentication_providers.with_raw_response.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
            },
            short_name="myorg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = response.parse()
        assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.authentication_providers.with_streaming_response.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
            },
            short_name="myorg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = response.parse()
            assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.retrieve(
            "id",
        )
        assert_matches_type(AuthenticationProviderRetrieveResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.authentication_providers.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = response.parse()
        assert_matches_type(AuthenticationProviderRetrieveResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.authentication_providers.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = response.parse()
            assert_matches_type(AuthenticationProviderRetrieveResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.authentication_providers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.update(
            id="id",
        )
        assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.update(
            id="id",
            active=True,
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
                "idp_cert_fingerprint_algorithm": "sha1",
            },
            settings_url="https://myorg.myidp.com/saml/metadata",
            short_name="myorg",
        )
        assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.authentication_providers.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = response.parse()
        assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.authentication_providers.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = response.parse()
            assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.authentication_providers.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.list()
        assert_matches_type(
            SyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.list(
            page_number=0,
            page_size=0,
            sort="name",
        )
        assert_matches_type(
            SyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.authentication_providers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.authentication_providers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        authentication_provider = client.authentication_providers.delete(
            "id",
        )
        assert_matches_type(AuthenticationProviderDeleteResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.authentication_providers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = response.parse()
        assert_matches_type(AuthenticationProviderDeleteResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.authentication_providers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = response.parse()
            assert_matches_type(AuthenticationProviderDeleteResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.authentication_providers.with_raw_response.delete(
                "",
            )


class TestAsyncAuthenticationProviders:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
            },
            short_name="myorg",
        )
        assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
                "idp_cert_fingerprint_algorithm": "sha256",
            },
            short_name="myorg",
            active=True,
            settings_url="https://myorg.myidp.com/saml/metadata",
        )
        assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.authentication_providers.with_raw_response.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
            },
            short_name="myorg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = await response.parse()
        assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.authentication_providers.with_streaming_response.create(
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
            },
            short_name="myorg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = await response.parse()
            assert_matches_type(AuthenticationProviderCreateResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.retrieve(
            "id",
        )
        assert_matches_type(AuthenticationProviderRetrieveResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.authentication_providers.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = await response.parse()
        assert_matches_type(AuthenticationProviderRetrieveResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.authentication_providers.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = await response.parse()
            assert_matches_type(AuthenticationProviderRetrieveResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.authentication_providers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.update(
            id="id",
        )
        assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.update(
            id="id",
            active=True,
            name="Okta",
            settings={
                "idp_cert_fingerprint": "13:38:C7:BB:C9:FF:4A:70:38:3A:E3:D9:5C:CD:DB:2E:50:1E:80:A7",
                "idp_entity_id": "https://myorg.myidp.com/saml/metadata",
                "idp_sso_target_url": "https://myorg.myidp.com/trust/saml2/http-post/sso",
                "idp_cert_fingerprint_algorithm": "sha1",
            },
            settings_url="https://myorg.myidp.com/saml/metadata",
            short_name="myorg",
        )
        assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.authentication_providers.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = await response.parse()
        assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.authentication_providers.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = await response.parse()
            assert_matches_type(AuthenticationProviderUpdateResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.authentication_providers.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.list()
        assert_matches_type(
            AsyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.list(
            page_number=0,
            page_size=0,
            sort="name",
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.authentication_providers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.authentication_providers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[AuthenticationProvider], authentication_provider, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        authentication_provider = await async_client.authentication_providers.delete(
            "id",
        )
        assert_matches_type(AuthenticationProviderDeleteResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.authentication_providers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication_provider = await response.parse()
        assert_matches_type(AuthenticationProviderDeleteResponse, authentication_provider, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.authentication_providers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication_provider = await response.parse()
            assert_matches_type(AuthenticationProviderDeleteResponse, authentication_provider, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.authentication_providers.with_raw_response.delete(
                "",
            )
